﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using GraphLayout;

namespace SilverTreeContainer
{
	public class SilverTreeContainer : Panel
	{
		#region Private fields
		LayeredTreeDraw _ltd;
		int _iNextNameSuffix = 0;
		Path _pthConnections = null;
		#endregion

		#region Dependency Properties

		#region Root
		public static readonly DependencyProperty RootProperty =
			DependencyProperty.Register(
				"Root",
				typeof(String),
				typeof(SilverTreeContainer),
				null
			);

		public string Root
		{
			get
			{
				return (string)GetValue(RootProperty);
			}
			set
			{
				SetValue(RootProperty, value);
			}
		}
		#endregion

		#region VerticalJustification
		public static readonly DependencyProperty VerticalJustificationProperty =
			DependencyProperty.Register(
				"VerticalJustification",
				typeof(VerticalJustification),
				typeof(SilverTreeContainer),
				null
			);

		public VerticalJustification VerticalJustification
		{
			get
			{
				return (VerticalJustification)GetValue(VerticalJustificationProperty);
			}
			set
			{
				SetValue(VerticalJustificationProperty, value);
			}
		}

		#endregion

		#region VerticalBufferProperty
		public static readonly DependencyProperty VerticalBufferProperty =
			DependencyProperty.Register(
				"VerticalBuffer",
				typeof(double),
				typeof(SilverTreeContainer),
				null
			);

		public double VerticalBuffer
		{
			get { return (double)GetValue(VerticalBufferProperty); }
			set { SetValue(VerticalBufferProperty, value); }
		}

		#endregion

		#region HorizontalBufferSubtreeProperty
		public readonly static DependencyProperty HorizontalBufferSubtreeProperty =
			DependencyProperty.Register(
				"HorizontalBufferSubtree",
				typeof(double),
				typeof(SilverTreeContainer),
				null
			);

		public double HorizontalBufferSubtree
		{
			get { return (double)GetValue(HorizontalBufferSubtreeProperty); }
			set { SetValue(HorizontalBufferSubtreeProperty, value); }
		}
		#endregion

		#region HorizontalBufferProperty
		public readonly static DependencyProperty HorizontalBufferProperty =
			DependencyProperty.Register(
				"HorizontalBuffer",
				typeof(double),
				typeof(SilverTreeContainer),
				null
			);

		public double HorizontalBuffer
		{
			get { return (double)GetValue(HorizontalBufferProperty); }
			set { SetValue(HorizontalBufferProperty, value); }
		}
		#endregion
		#endregion

		#region Parenting
		private void SetParents(TreeNode tnRoot)
		{
			// First pass to clear all parents
			foreach (UIElement uiel in Children)
			{
				TreeNode tn = uiel as TreeNode;
				if (tn != null)
				{
					tn.ClearParent();
				}
			}

			// Second pass to properly set them from their children...
			foreach (UIElement uiel in Children)
			{
				TreeNode tn = uiel as TreeNode;
				if (tn != null && tn != tnRoot)
				{
					tn.SetParent();
				}
			}
		}
		#endregion

		#region Public utilities
		public void Clear()
		{
			Children.Clear();
			_pthConnections = null;
		}

		private void SetName(TreeNode tn, string strName)
		{
			tn.Name = strName;
			tn.SetValue(Panel.NameProperty, strName);
		}

		public TreeNode AddRoot(Object objContent, string strName)
		{
			TreeNode tnNew = new TreeNode();
			SetName(tnNew, strName);
			tnNew.Content = objContent;
			Children.Add(tnNew);
			Root = strName;
			return tnNew;
		}

		public TreeNode AddRoot(Object objContent)
		{
			return AddRoot(objContent, StrNextName());
		}

		public TreeNode AddNode(Object objContent, string strName, string strParent)
		{
			TreeNode tnNew = new TreeNode();
			SetName(tnNew, strName);
			tnNew.Content = objContent;
			tnNew.TreeParent = strParent;
			Children.Add(tnNew);
			return tnNew;
		}

		private string StrNextName()
		{
			return "__TreeNode" + _iNextNameSuffix++;
		}

		public TreeNode AddNode(Object objContent, string strName, TreeNode tnParent)
		{
			return AddNode(objContent, strName, tnParent.Name);
		}

		public TreeNode AddNode(Object objContent, TreeNode tnParent)
		{
			return AddNode(objContent, StrNextName(), tnParent.Name);
		}
		#endregion

		#region Panel Overrides
		protected override Size MeasureOverride(Size availableSize)
		{
			if (Children.Count == 0)
			{
				return new Size(100, 20);
			}

			if (_pthConnections != null && Children.Contains(_pthConnections))
			{
				Children.Remove(_pthConnections);
				_pthConnections = null;
			}

			Size szFinal = new Size(0, 0);
			string strRoot = Root;
			TreeNode tnRoot = this.FindName(strRoot) as TreeNode;

			foreach (UIElement uiel in Children)
			{
				uiel.Measure(availableSize);
				Size szThis = uiel.DesiredSize;

				if (szThis.Width > szFinal.Width || szThis.Height > szFinal.Height)
				{
					szFinal = new Size(
						Math.Max(szThis.Width, szFinal.Width),
						Math.Max(szThis.Height, szFinal.Height));
				}
			}

			if (tnRoot != null)
			{
				SetParents(tnRoot);
				_ltd = new LayeredTreeDraw(tnRoot, HorizontalBuffer, HorizontalBufferSubtree, VerticalBuffer, VerticalJustification.top);
				_ltd.LayoutTree();
				szFinal = new Size(_ltd.PxOverallWidth, _ltd.PxOverallHeight);
			}

			// Put in the connections too...
			if (_ltd.Connections != null)
			{
				_pthConnections = new Path();
				PathGeometry pg = new PathGeometry();
				_pthConnections.Stroke = new SolidColorBrush(Colors.Black);
				_pthConnections.StrokeThickness = 1.0;
				pg.Figures = new PathFigureCollection();

				foreach (TreeConnection tcn in _ltd.Connections)
				{
					PathFigure pf = new PathFigure();

					pf.StartPoint = PtFromDPoint(tcn.LstPt[0]);
					pf.IsClosed = false;
					pf.Segments = new PathSegmentCollection();
					for (int iPt = 1; iPt < tcn.LstPt.Count; iPt++)
					{
						LineSegment ls = new LineSegment();
						ls.Point = PtFromDPoint(tcn.LstPt[iPt]);
						pf.Segments.Add(ls);
					}
					pg.Figures.Add(pf);
				}
				_pthConnections.Data = pg;
				Children.Add(_pthConnections);
				_pthConnections.Measure(availableSize);
			}
			return szFinal;
		}

		static Point PtFromDPoint(DPoint dpt)
		{
			return new Point(dpt.X, dpt.Y);
		}

		protected override Size ArrangeOverride(Size finalSize)
		{
			foreach (UIElement uiel in Children)
			{
				TreeNode tn = uiel as TreeNode;
				Point ptLocation = new Point(0, 0);
				if (tn != null)
				{
					ptLocation = new Point(_ltd.X(tn), _ltd.Y(tn));
				}
				uiel.Arrange(new Rect(ptLocation, uiel.DesiredSize));
			}

			return finalSize;
		}
		#endregion

	}
}
